#ifndef INC_GTSTRINPUT_H
#define INC_GTSTRINPUT_H

#include "gtwidget.h"

/************************************************************************\
*
* GtStrInput class header.
*
* $Ident: This file is part of the 'Gt-library' 1.0.0 $
* $Copyright: (C) 1997 by Gran Gustafsson $
*
\************************************************************************/

/* Type of callback data for edit */
typedef struct GtSIEditCD
{
	int why;
	XEvent *xevent;
	KeySym key;
	ulong state;
	ulong currpos,newpos;
	char *ins;
	int len;
	sbool doedit;
} GtSIEditCD;

/* Type of callback data for done */
typedef struct GtSIDoneCD
{
	int why;
	char *str;
	int len;
} GtSIDoneCD;

/* Why edit */
#define GTEW_TYPED		0		/* Typed string */
#define GTEW_PASTE		1		/* Pasted string */
#define GTEW_MOVE			2		/* Movement */
#define GTEW_BSPACE		3		/* Backspace */
#define GTEW_DELETE		4		/* Delete string */
#define GTEW_RPLACE		5		/* Replace string */
#define GTEW_DONE			6		/* Done editing */

/* Why done */
#define GTDW_UPTAB		0		/* Shift + Tab */
#define GTDW_DOWNTAB	1		/* Tab */
#define GTDW_UP				2		/* Cursor up */
#define GTDW_DOWN			3		/* Cursor down */
#define GTDW_ENTER		4		/* Return */
#define GTDW_MOUSE		5		/* Deselect by mouse */

class GtStrInput : public GtWidget
{
public:
		/* Standard */
	GtStrInput(void);
	void Destroy(void);
	sbool Init(char *n,GtManager *p,GtTag *tgs);
	sbool Set(GtTag *tgs);
	void Get(GtTag *tgs);
	char *GetClass(void) { return("GtStrInput"); }
	sbool IsSubClassOf(char *cn);
		/* Events */
	int Handle(XEvent *xe);
		/* Values */
	void SetEMask(ulong em);
		/* Notify */
	void Notify(ulong tag);
		/* Callbacks */
	void AddCB(ulong tag,GtCallB cb,GtUDPtr ud);
	void RemCB(ulong tag,GtCallB cb);
		/* String management */
	ulong GetCursor(void) { return(pos); }
	char *GetString(void) { return(str); }
	char *GetMark(void) { return(mark); }
	void SetCursor(ulong npos);
	void InsertString(char *istr);
	void ReplaceString(char *rstr);
	void DeleteString(ulong dlen);
	void MarkString(ulong start,ulong end);
	void Activate(void);
	void Deactivate(void);
		/* Internal method */
	void SetPosSize(word xp,word yp,word xs,word ys);
protected:
		/* Internal methods */
	sbool Draw(XEvent *xe);
	sbool Key(XEvent *xe);
	sbool Mouse(XEvent *xe);
private:
	void DoKey(GtSIEditCD *cd);
	void MoveCurs(ulong npos);
	void InsertStr(char *str);
	void DeleteStr(ulong dl);
	void ReDraw(ulong fst,ulong lst);
	void AllocMark(void);
	void FreeMark(void);
	static int MarkConv(GtBase *obj,GtCDPtr cd,GtUDPtr ud);
	static int MarkLose(GtBase *obj,GtCDPtr cd,GtUDPtr ud);
	static int MarkDone(GtBase *obj,GtCDPtr cd,GtUDPtr ud);
	static int Paste(GtBase *obj,GtCDPtr cd,GtUDPtr ud);
protected:
		/* Instance variables */
	GtCBList edit,done;
	XRectangle rect;
	char *str,*ins,*mark;
	ulong maxl,len,pos,first,mvis;
	ulong mstart,mend;
	Time last;
	Atom paste;
	word foff,fh,mw;
	ubyte ftype,siflags,ibw,ibdrw,bdrw;
};

extern GtStrInput *GtMkStrInput(char *n,GtManager *p,GtTag *tgs);
extern GtStrInput *GtMkStrInput(char *n,GtManager *p,ulong tag,...);

#endif /* INC_GTSTRINPUT_H */
