#include <proto.h>

#include "gtpopup.h"
#include "gterror.h"
#include "gtcommon.h"

/************************************************************************\
*
* GtPopup class.
*
* $Ident: This file is part of the 'Gt-library' 1.0.0 $
* $Copyright: (C) 1997 by Gran Gustafsson $
*
\************************************************************************/

#define GTPG_MASK		(ButtonPressMask|ButtonReleaseMask|PointerMotionMask|\
								PointerMotionHintMask|Button1MotionMask|Button2MotionMask|\
								Button3MotionMask|Button4MotionMask|Button5MotionMask|\
								ButtonMotionMask|EnterWindowMask|LeaveWindowMask)

/* Internal flags for GtPopup */
#define GTPF_GRAB			0x01


/************************************************************************\
*
* Create and init popup.
*
\************************************************************************/
GtPopup *GtMkPopup(char *n,GtManager *p,GtTag *tgs)
{
	GtPopup *obj;
	
	if(!(obj=new GtPopup()))
	{
		GtError(NULL,GTEC_NOOBJECT,"No GtPopup object!\n");
		return(NULL);
	}
	if(!obj->Init(n,p,tgs))
	{
		GtDestroy(obj);
		obj=NULL;
	}
	return(obj);
}
	
/************************************************************************\
*
* Create and init popup.
*
\************************************************************************/
GtPopup *GtMkPopup(char *n,GtManager *p,ulong tag,...)
{
	va_list va;
	GtPopup *obj;
	int ps;
	
	va_start(va,tag);
	ps=GtBeginVaTags();
	obj=GtMkPopup(n,p,GtCopyVaTags(tag,&va));
	GtEndVaTags(ps);
	va_end(va);
	return(obj);
}
	
/************************************************************************\
*
* Setup empty popup.
*
\************************************************************************/
GtPopup::GtPopup(void)
{
	popflags=0;
}

/************************************************************************\
*
* Destroy popup.
*
\************************************************************************/
void GtPopup::Destroy(void)
{
	if(popflags & GTPF_GRAB)
	{
		this->UnGrab();
		dsp->ClrGrab(this);
	}
	GtShell::Destroy();
}

/************************************************************************\
*
* Check if class is (sub)class of named class.
*
\************************************************************************/
sbool GtPopup::IsSubClassOf(char *cn)
{
	if(strcmp(cn,"GtPopup")==0)
		return(TRUE);
	else
		return(GtShell::IsSubClassOf(cn));
}

/************************************************************************\
*
* Map or unmap popup.
*
\************************************************************************/
void GtPopup::Map(sbool map)
{
	GtShell::Map(map);
	if(popflags & GTPF_GRAB)
	{
		if(flags & GTWF_MAPPED)
		{
			this->Grab();
			dsp->SetGrab(this);
		}
		else
		{
			this->UnGrab();
			dsp->ClrGrab(this);
		}	
	}
}
	
/************************************************************************\
*
* Build popup and set special window attributes.
*
\************************************************************************/
sbool GtPopup::Build(void)
{
	if(flags & GTWF_UNBUILT)
	{
		if(!gtwin->SetV(GT_Override,TRUE,GT_Raised,TRUE,GT_DONE))
			return(FALSE);
	}
	return(GtShell::Build());
}

/************************************************************************\
*
* Set popup event mask.
*
\************************************************************************/
void GtPopup::SetEMask(ulong em)
{
	GtShell::SetEMask(em|StructureNotifyMask);
}

/************************************************************************\
*
* Show popup at given position with or without pointer grab.
*
\************************************************************************/
void GtPopup::Popup(int xp,int yp,sbool grab,Time tm)
{
	if(flags & GTWF_UNBUILT)
		return;
	gtime=tm;
	gtwin->Move(xp,yp);
	if(grab)
		popflags|=GTPF_GRAB;
	this->Map(TRUE);
}

/************************************************************************\
*
* Show popup at it's current position with or without pointer grab.
*
\************************************************************************/
void GtPopup::Popup(sbool grab,Time tm)
{
	if(flags & GTWF_UNBUILT)
		return;
	gtime=tm;
	if(grab)
		popflags|=GTPF_GRAB;
	this->Map(TRUE);
}

/************************************************************************\
*
* Hide popup.
*
\************************************************************************/
void GtPopup::Popdown(void)
{
	this->Map(FALSE);
}

/************************************************************************\
*
* Handle popup map notifies.
*
\************************************************************************/
sbool GtPopup::Config(XEvent *xe)
{
	sbool ret;
	
	ret=GtShell::Config(xe);
	if(xe && (xe->type==MapNotify))
	{
		ret=TRUE;
		if(popflags & GTPF_GRAB)
			this->Grab();
	}
	return(ret);
}

/************************************************************************\
*
* Grab input.
*
\************************************************************************/
void GtPopup::Grab(void)
{
	if((popflags & GTPF_GRAB) && !(flags & GTWF_UNBUILT))
	{
		gtwin->GrabPointerV(gtime,
			GT_EventMask,this->GetEMask()&GTPG_MASK,
			GT_DONE);
	}
}

/************************************************************************\
*
* Ungrab input.
*
\************************************************************************/
void GtPopup::UnGrab(void)
{
	if((popflags & GTPF_GRAB) && !(flags & GTWF_UNBUILT))
		gtwin->UnGrabPointer(gtime);
}
